import pandas as pd


class Metric:
    def __init__(self):
        pass

    def add_metric(self, name, default=0):
        pass

    def add_row(self, metric_dict=None, name=None, value=None, index=None):
        pass

